@@ -161,7 +161,7 @@ import ast
 from copy import deepcopy
 import os
 import pathlib
-from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 from ae.base import norm_line_sep, norm_name                                    # type: ignore
 from ae.core import DEBUG_LEVEL_VERBOSE                                         # type: ignore
@@ -169,7 +169,7 @@ from ae.i18n import get_f_string
 from ae.gui_app import id_of_flow, flow_action, flow_key, replace_flow_action   # type: ignore
 
 
-__version__ = '0.1.26'
+__version__ = '0.1.27'
 
 
 FLOW_PATH_ROOT_ID = 'ROOT'  #: pseudo item id needed for flow path jumper and for drop onto leave item button
@@ -225,6 +225,34 @@ def correct_item_id(item_id: str) -> str:
     return item_id.lstrip(INVALID_ITEM_ID_PREFIX_CHARS)
 
 
+def item_sel_filter(item: LiszItem) -> bool:
+    """ callable for to filter selected LiszItems.
+
+    :param item:        item data structure to check.
+    :return:            True if item is a selected leaf or if item is a node with only selected sub-leaves, else False.
+    """
+    if 'node' in item:
+        for sub_item in item['node']:
+            if not item_sel_filter(sub_item):
+                return False
+        return True
+    return item.get('sel') == 1
+
+
+def item_unsel_filter(item: LiszItem) -> bool:
+    """ callable for to filter unselected LiszItems.
+
+    :param item:        item data structure to check.
+    :return:            True if item is an unselected leaf or if item is a node with only unselected leaves, else False.
+    """
+    if 'node' in item:
+        for sub_item in item['node']:
+            if not item_unsel_filter(sub_item):
+                return False
+        return True
+    return item.get('sel', 0) == 0
+
+
 def node_from_literal(lit: str) -> LiszNode:
     """ parse passed string for to integrate/add them into the current displayed node or item.
 
@@ -250,24 +278,6 @@ def node_from_literal(lit: str) -> LiszNode:
     return node
 
 
-def item_sel_filter(item: LiszItem) -> bool:
-    """ callable for to filter selected LiszItems.
-
-    :param item:        item data structure to check.
-    :return:            True if item is selected.
-    """
-    return bool(item.get('sel', False)) is True
-
-
-def item_unsel_filter(item: LiszItem) -> bool:
-    """ callable for to filter selected LiszItems.
-
-    :param item:        item data structure to check.
-    :return:            True if item is not selected.
-    """
-    return bool(item.get('sel', False)) is False
-
-
 class LiszDataMixin(ABC):
     """ lisz data model - independent from used GUI framework. """
     root_node: LiszNode             #: root of lisz data structure
@@ -295,7 +305,7 @@ class LiszDataMixin(ABC):
         """ add debug sound on each state change/save (ae.gui_app.MainAppBase method). """
 
     @abstractmethod
-    def change_flow(self, new_flow_id: str, popups_to_close: Sequence = (), **event_kwargs) -> bool:
+    def change_flow(self, new_flow_id: str, **event_kwargs) -> bool:
         """ change/switch flow id. """
 
     @abstractmethod
@@ -317,31 +327,53 @@ class LiszDataMixin(ABC):
 
     # helper methods
 
-    def add_item(self, nid: LiszItem) -> str:
-        """ add item to currently displayed node.
+    def add_item(self, nid: LiszItem, node_to_add_to: Optional[LiszNode] = None, new_item_index: int = 0) -> str:
+        """ add item (leaf or node) to currently displayed node.
 
         :param nid:             LiszItem to add (has to have a non-empty item id).
+        :param node_to_add_to:  node where the passed item will be added to (def=current node).
+        :param new_item_index:  index where the new item will be inserted (default=0, ignored if item already exists).
         :return:                error message if any error happened, else empty string.
         """
+        if node_to_add_to is None:
+            node_to_add_to = self.current_node_items
+
         item_id = nid['id']
         want_node = 'node' in nid
-        err_msg = self.edit_validate(-1, item_id, want_node=want_node)
-        if not err_msg and want_node:
-            self.current_node_items[0]['node'].extend(nid['node'])
+        err_msg = self.edit_validate(-1, item_id, want_node=want_node,
+                                     parent_node=node_to_add_to, new_item_index=new_item_index)
+        if want_node:
+            if err_msg:
+                item_idx = self.find_item_index(item_id, searched_node=node_to_add_to)
+                if item_idx >= 0:                                   # found the blocking/duplicate item id
+                    if 'node' not in node_to_add_to[item_idx]:
+                        node_to_add_to[item_idx]['node'] = list()   # convert blocking item from leaf to node
+                    err_msg = get_f_string("(ignorable) ") + err_msg
+                    sub_err = self.add_items(nid['node'], node_to_add_to=node_to_add_to[item_idx]['node'])
+                    if sub_err:                                     # add (ignorable?) errors from sub-node adds
+                        err_msg += "\n" + sub_err
+            else:
+                node_to_add_to[new_item_index]['node'].extend(nid['node'])
+
         return err_msg
 
-    def add_items(self, items: LiszNode) -> str:
+    def add_items(self, items: LiszNode, node_to_add_to: Optional[LiszNode] = None) -> str:
         """ add item to currently displayed node.
 
         :param items:           LiszNode list to add (each item has to have a non-empty item id).
+        :param node_to_add_to:  node where the passed item will be added to (def=current node).
         :return:                error message if any error happened (multiple error messages are separated by \\\\n),
                                 else empty string.
         """
+        if node_to_add_to is None:
+            node_to_add_to = self.current_node_items
+
         errors = list()
-        for item in items:
-            err_msg = self.add_item(item)
+        for item in reversed(items):
+            err_msg = self.add_item(item, node_to_add_to=node_to_add_to)
             if err_msg:
                 errors.append(err_msg)
+
         return "\n".join(errors)
 
     def change_sub_node_sel(self, node: LiszNode, set_sel_to: bool):
@@ -390,17 +422,17 @@ class LiszDataMixin(ABC):
                       parent_node: Optional[LiszNode] = None, new_item_index: int = 0) -> str:
         """ validate the user changes after adding a new item or editing an existing item.
 
-        :param old_item_index:      index in the current node of the edited item or -1 if a new item (to be added).
-        :param new_id:              new/edited id string.
-        :param want_node:           True if the new/edited item will have a sub-node, False if not.
-        :param parent_node:         node where the edited/added item as to be updated/inserted (default=current list).
-        :param new_item_index:      index where the new item have to be inserted (default=0, ignored in edit item mode).
-        :return:                    empty string on successful edit validation or on cancellation of new item (with
-                                    empty id string), else error string or
-                                    `'request_delete_confirmation_for_item'` if the user has to confirm the deletion
-                                    after the user wiped the item id string or
-                                    `'request_delete_confirmation_for_node'` if the user has to confirm the
-                                    removal of the sub-node.
+        :param old_item_index:  index in the current node of the edited item or -1 if a new item (to be added).
+        :param new_id:          new/edited id string.
+        :param want_node:       True if the new/edited item will have a sub-node, False if not.
+        :param parent_node:     node where the edited/added item as to be updated/inserted (default=current list).
+        :param new_item_index:  index where the new item have to be inserted (default=0, ignored in edit item mode).
+        :return:                empty string on successful edit validation or on cancellation of new item (with
+                                empty id string), else error string or
+                                `'request_delete_confirmation_for_item'` if the user has to confirm the deletion
+                                after the user wiped the item id string or
+                                `'request_delete_confirmation_for_node'` if the user has to confirm the
+                                removal of the sub-node.
         """
         add_item = old_item_index == -1
         if parent_node is None:
@@ -577,12 +609,14 @@ class LiszDataMixin(ABC):
 
         return paths
 
-    def flow_path_text(self, flow_path: List[str], min_len: int = 3, display_root: bool = False) -> str:
+    def flow_path_text(self, flow_path: List[str], min_len: int = 3, display_root: bool = False,
+                       separator: str = FLOW_PATH_TEXT_SEP) -> str:
         """ generate shortened display text from the passed flow path.
 
         :param flow_path:       flow path list.
         :param min_len:         minimum length of node ids (flow id keys).
         :param display_root:    pass True to return FLOW_PATH_ROOT_ID on empty/root path.
+        :param separator:       path item separator (default=FLOW_PATH_TEXT_SEP).
         :return:                shortened display text string of the passed flow path (which can be converted back
                                 to a flow path list with the method :meth:`.flow_path_from_text`).
         """
@@ -591,7 +625,7 @@ class LiszDataMixin(ABC):
         node = self.root_node
         for flow_id in flow_path:
             if flow_action(flow_id) != 'enter':
-                break
+                continue
             id_len = min_len
             node_id = flow_key(flow_id)
             sub_id_len = len(node_id)
@@ -606,7 +640,7 @@ class LiszDataMixin(ABC):
             if path_nid and 'node' in path_nid:     # prevent error in quick jump to root
                 node = path_nid['node']             # type: ignore   # mypy not supports recursive types see issue #731
 
-        return FLOW_PATH_TEXT_SEP.join(shortened_ids) if shortened_ids else (FLOW_PATH_ROOT_ID if display_root else '')
+        return separator.join(shortened_ids) if shortened_ids else (FLOW_PATH_ROOT_ID if display_root else '')
 
     def focus_neighbour_item(self, delta: int):
         """ move flow id to previous/next displayed/filtered node item.
@@ -756,10 +790,12 @@ class LiszDataMixin(ABC):
 
         return True
 
-    def node_info(self, node: LiszNode, what: Tuple[str, ...] = ()) -> Dict[str, Union[int, str, List[str]]]:
+    def node_info(self, node: LiszNode, what: Tuple[str, ...] = (), recursive: bool = True
+                  ) -> Dict[str, Union[int, str, List[str]]]:
         """ determine statistics info for the node specified by :paramref:`~node_info.flow_path`.
 
         :param node:            node to get info for.
+
         :param what:            pass tuple of statistic info fields for to include only these into the returned dict
                                 (passing an empty tuple or nothing will include all the following fields):
 
@@ -773,15 +809,17 @@ class LiszDataMixin(ABC):
                                 * 'selected_leaf_names': list of all selected sub-leaf names.
                                 * 'unselected_leaf_names': list of all unselected sub-leaf names.
 
+        :param recursive:       pass False if only the passed node has to be investigated.
+
         :return:                dict with the node info specified by the :paramref:`~node_info.what` argument.
         """
-        names = self.sub_item_ids(node=node, leaves_only=False)
+        names = self.sub_item_ids(node=node, leaves_only=False, recursive=recursive)
         count = len(names)
-        leaf_names = self.sub_item_ids(node=node)
+        leaf_names = self.sub_item_ids(node=node, recursive=recursive)
         leaf_count = len(leaf_names)
-        selected_leaf_names = self.sub_item_ids(node=node, hide_sel_val=False)
+        selected_leaf_names = self.sub_item_ids(node=node, hide_sel_val=False, recursive=recursive)
         selected_leaf_count = len(selected_leaf_names)          # noqa: F841
-        unselected_leaf_names = self.sub_item_ids(node=node, hide_sel_val=True)
+        unselected_leaf_names = self.sub_item_ids(node=node, hide_sel_val=True, recursive=recursive)
         unselected_leaf_count = len(unselected_leaf_names)      # noqa: F841
         node_count = count - leaf_count                         # noqa: F841
 
@@ -927,32 +965,40 @@ class LiszDataMixin(ABC):
         if node:
             self.change_sub_node_sel(node, set_sel_to)
         else:
-            # assert not set_sel_to == bool(self.current_node_items[node_idx].get('sel', 0))
+            # assert not set_sel_to == bool(self.current_node_items[node_idx].get('sel'))
             self.toggle_item_sel(node_idx)
         event_kwargs['flow_id'] = id_of_flow('focus', 'item', item_id)
         return True
 
     def on_node_extract(self, flow_path_text: str, event_kwargs: dict) -> bool:
-        """ FlowButton tap event handler for to extract (copy/cut/delete/export) the node specified by `flow_path_text`.
+        """ extract the leaves of the node specified by `flow_path_text`.
 
-        :param flow_path_text:  flow path text or list literal (identifying the node to extract).
-        :param event_kwargs:    event arguments (used for to pass the `export_path` and `extract_type`):
+        :param flow_path_text:  flow path text or list literal (identifying the start node to extract from).
 
-                                `export_path` specifies the destination folder of the export (default='.'/CWD).
+        :param event_kwargs:    extra arguments specifying extract options (only `extract_type` is mandatory):
 
                                 `extract_type` specifies extract destination and an optional filter on un-/selected
-                                items, e.g. the following string values can be passed for a 'copy' destination extract:
+                                items. The first part defines the extract action (copy/cut/delete/export/share)
+                                and an optional second part (separated by an underscore) the filter.
+                                E.g. the following string values can be passed for a 'copy' extract action:
 
                                 * 'copy' is copying all items of the specified node to the clipboard.
                                 * 'copy_sel' is copying only the selected items of the node to the clipboard.
                                 * 'copy_unsel' is copying only the unselected items to the clipboard.
 
+                                `recursive` specifies if `False` that the extraction affects only the leaves of the
+                                current node specified by `flow_path_text` and
+                                if `True` the extraction affects also the leaves of the sub-nodes (default=True).
+
+                                `export_path` specifies the destination folder of the export action (default='.'/CWD).
+
         :return:                True for to process/change flow.
         """
         flow_path = ast.literal_eval(flow_path_text) if flow_path_text and flow_path_text[0] == '[' else \
             self.flow_path_from_text(flow_path_text)
         node = self.flow_path_node(flow_path)
-        dst, *what = event_kwargs['extract_type'].split('_')
+        extract_action, *what = event_kwargs['extract_type'].split('_')
+        recursive = event_kwargs.get('recursive', True)
 
         if not what:
             extract_filter = None
@@ -964,19 +1010,32 @@ class LiszDataMixin(ABC):
             extract_filter = item_sel_filter
             delete_filter = item_unsel_filter
 
-        extract_node = deepcopy(node)
-        if dst in ('cut', 'delete'):
-            self.shrink_node_size(node, item_filter=delete_filter)      # in-place deletion
-        self.shrink_node_size(extract_node, item_filter=extract_filter)
+        extract_node = deepcopy(node) if recursive else [item for item in node if 'node' not in item]
+        snd_name = 'added'
+        if extract_action in ('cut', 'delete'):
+            self.shrink_node_size(node, item_filter=delete_filter, recursive=recursive)      # in-place deletion
+            snd_name = 'deleted'
+            event_kwargs['flow_id'] = id_of_flow('')
+            event_kwargs['reset_last_focus_flow_id'] = True
+        self.shrink_node_size(extract_node, item_filter=extract_filter, recursive=recursive)
 
-        if dst in ('copy', 'cut'):
+        if extract_action in ('copy', 'cut'):
             self.call_method('on_clipboard_key_c', repr(extract_node))  # Clipboard.copy(repr(node))
-        elif dst == 'export':
+        elif extract_action == 'export':
             self.export_node(flow_path, file_path=event_kwargs.get('export_path', '.'), node=extract_node)
+        elif extract_action == 'share':
+            self.call_method('share_node', flow_path, node=extract_node)
 
+        event_kwargs['changed_event_name'] = 'refresh_all'
+
+        # dismiss prevents dropdown._reposition() weakref-err (refresh_all() recreates attach_to widget)
+        # event_kwargs['popups_to_close'][0].attach_to = None  # close() results in weakref-err in container.on_dismiss
+        # now fixed by resetting attach_to to None in MainAppBase.change_flow()
+
+        self.play_sound(snd_name)
         return True
 
-    def on_node_jump(self, flow_path_text: str, event_kwargs: dict) -> bool:
+    def on_node_jump(self, flow_path_text: str, event_kwargs: Dict[str, Any]) -> bool:
         """ FlowButton clicked event handler restoring flow path from the flow key.
 
         Used for to jump to node specified by the flow path text in the passed flow_id key.
@@ -1037,33 +1096,36 @@ class LiszDataMixin(ABC):
         """ refresh current node including the depending display node. """
         self.current_node_items = self.flow_path_node()
 
-    def shrink_node_size(self, node: LiszNode, item_filter: Optional[Callable[[LiszItem], bool]] = None):
-        """ shrink node size by removing unneeded items and `sel` keys, e.g. for to save space in config file.
+    def shrink_node_size(self, node: LiszNode, item_filter: Optional[Callable[[LiszItem], bool]] = None,
+                         recursive: bool = True):
+        """ shrink node size by removing unneeded items and `sel` keys, e.g. for to export or save space in config file.
 
         :param node:            start or root node to shrink (in-place!).
         :param item_filter:     pass callable for to remove items from the passed node and its sub-nodes.
                                 The callable is getting passed each item as argument and has to return True
                                 for to remove it from its node.
+        :param recursive:       pass False if only the passed start node has to be shrunk.
         """
-
         del_items = list()
+
         for item in node:
             is_node = 'node' in item
-            if not is_node and item_filter and item_filter(item):
-                del_items.append(item)
-            elif item.get('sel', 1) != 1:
-                item.pop('sel')
+            if is_node or item.get('sel', 0) == 0:
+                item.pop('sel', None)
             elif 'sel' in item:
-                item['sel'] = 1  # remove also decimal point and zero (float to int)
-            if is_node:
+                item['sel'] = 1  # remove also decimal point and zero (float to int) from this selected leaf item
+            if recursive and is_node:
                 self.shrink_node_size(item['node'], item_filter=item_filter)
+            if item_filter and item_filter(item):
+                del_items.append(item)
+
         for item in del_items:
             node.remove(item)
 
     def sub_item_ids(self, node: Optional[LiszNode] = None, item_id: str = '',
-                     leaves_only: bool = True, hide_sel_val: Optional[bool] = None,
+                     leaves_only: bool = True, hide_sel_val: Optional[bool] = None, recursive: bool = True,
                      sub_ids: Optional[List[str]] = None) -> List[str]:
-        """ return item names of all items including their sub-node items (if exists).
+        """ return item names of all items including their sub-node items (if exists and recursive==True).
 
         Used for to determine the affected items if user want to delete or de-/select the sub-items of
         the node item specified by the :paramref:`sub_item_ids.item_id` argument.
@@ -1076,6 +1138,7 @@ class LiszDataMixin(ABC):
         :param leaves_only:     True (the default) if only include the leaf item ids of the sub-nodes of the node item.
         :param hide_sel_val:    pass False/True for to exclude un-/selected items from the returned list of ids.
                                 If None or not passed then all found items will be included.
+        :param recursive:       pass False if only the passed start node has to be investigated/included.
         :param sub_ids:         already found sub item ids (used only for the recursive calls of this method).
         :return:                list of found item ids.
         """
@@ -1087,9 +1150,9 @@ class LiszDataMixin(ABC):
             sub_ids = list()
 
         for item in node:           # type: ignore # mypy does not recognize that node cannot be None
-            if not leaves_only if 'node' in item else (hide_sel_val is None or bool(item.get('sel')) != hide_sel_val):
+            if (not leaves_only) if 'node' in item else (hide_sel_val is None or bool(item.get('sel')) != hide_sel_val):
                 sub_ids.append(item['id'])
-            if item.get('node'):
+            if recursive and item.get('node'):
                 self.sub_item_ids(node=node, item_id=item['id'], leaves_only=leaves_only, hide_sel_val=hide_sel_val,
                                   sub_ids=sub_ids)
 
@@ -1101,6 +1164,6 @@ class LiszDataMixin(ABC):
         :param node_idx:            list index of the item in the current node to change the selection for.
         """
         if item_sel_filter(self.current_node_items[node_idx]):
-            self.current_node_items[node_idx].pop('sel')
+            self.current_node_items[node_idx].pop('sel', None)
         else:
             self.current_node_items[node_idx]['sel'] = 1.0
